using Microsoft.Xna.Framework;
using Xfinity.AI.Entities.Managers;

namespace Xfinity.Physics
{
    public class PhysicsManager : GlobalManager
    {
        /// <summary>
        /// Initializes a new instance of the <see cref="PhysicsManager"/> class.
        /// </summary>
        /// <param name="game">Game the game component should be attached to.</param>
        public PhysicsManager(Game game)
            : base("Physics manager", false,game)
        {
        }
        
        /// <summary>
        /// The gravitational constant (G).
        /// </summary>
        public const double GravitationalConstant = 6.6742e-11;

        /// <summary>
        /// Planks Constant in JouleSeconds.
        /// </summary>
        public const double PlanksConstantInJouleSeconds = 6.6260693e-34;

        /// <summary>
        /// Planks Constant in electron-VoltSeconds.
        /// </summary>
        public const double PlanksConstantInElectronVoltSeconds = 4.13566743e-15;

        /// <summary>
        /// Dirac's Constant in JouleSeconds.
        /// </summary>
        public const double DiracConstantInJouleSeconds = PlanksConstantInJouleSeconds / MathHelper.TwoPi;

        /// <summary>
        /// Dirac's Constant in electron-VoltSeconds.
        /// </summary>
        public const double DiracConstantInElectronVoltSecond = PlanksConstantInElectronVoltSeconds / MathHelper.TwoPi;

        /// <summary>
        /// Boltzmann's Constant.
        /// </summary>
        public const double BoltzmannConstantInJouleKelvins = 1.3806505e-23;

        /// <summary>
        /// Boltzmann's Constant.
        /// </summary>
        public const double BoltzmannConstantInElectronVoltSeconds = 8.617343e-5;

        private static Vector3 g = new Vector3(0, -9.81f, 0);

        public static Vector3 GlobalGravity
        {
            get { return g; }
            set { g = value; }
        }
    }
}
