using System;
using Microsoft.Xna.Framework;
using Xfinity.AI.Entities.Managers;

namespace Xfinity.Player
{
    public class PlayerManager : GlobalManager
    {
        /// <summary>
        /// Initializes a new instance of the <see cref="PlayerManager"/> class.
        /// </summary>
        /// <param name="playerName">Name of the player.</param>
        /// <param name="game">The game.</param>
        public PlayerManager(string playerName, Game game)
            : base(
            (String.IsNullOrEmpty(playerName) ? 
#if !XBOX
            Properties.Settings.Default.PlayerName 
#else
            ""
#endif
            : playerName
            ) + " Manager", false, game)
        {
            #if !XBOX
            if (String.IsNullOrEmpty(playerName))
            {
                playerName = Properties.Settings.Default.PlayerName;
            }
            if (String.IsNullOrEmpty(Properties.Settings.Default.PlayerName) && String.IsNullOrEmpty(playerName))
            {
                Properties.Settings.Default.PlayerName = Environment.UserName;
                Properties.Settings.Default.Save();
                playerName = Properties.Settings.Default.PlayerName;
            }
            ;
#endif
            player = new Player(playerName, game);
        }
        /// <summary>
        /// Gets the name of the player.
        /// </summary>
        /// <value>The name of the player.</value>
        public string PlayerName
        {
            get { return player.Name; }
        }
        private Player player;

        /// <summary>
        /// Gets the player.
        /// </summary>
        /// <value>The player.</value>
        public Player Player
        {
            get { return player; }
        }

    }
}
