using System.Diagnostics;
using System.IO;

namespace Xfinity.Scripting
{
    /// <summary>
    /// A script to run.
    /// </summary>
    public class Script
    {
        private Script() { }
        /// <summary>
        /// Constructs a new Script object from a string.
        /// </summary>
        /// <param name="script">The script code.</param>
        /// <returns>A new script.</returns>
        [DebuggerStepThrough]
        public static Script FromString(string script)
        {
            Script s = new Script();
            s.code = script;
#if !XBOX
            ScriptManager.Scripts.Add(s);
#endif
            return s;
        }
        /// <summary>
        /// Constructs a new Script by reading from a file.
        /// </summary>
        /// <param name="filename">The filename to read.</param>
        /// <returns>A new script.</returns>
        [DebuggerStepThrough]
        public static Script FromFile(string filename)
        {
            Script s = new Script();
            s.code = File.ReadAllText(filename);
            #if !XBOX
            ScriptManager.Scripts.Add(s);
#endif
            return s;
        }
        private string code;

        /// <summary>
        /// The script's source code.
        /// </summary>
        public string Code
        {
            [DebuggerStepThrough]
            get { return code; }
            [DebuggerStepThrough]
            set { code = value; }
        }
        /// <summary>
        /// Returns the source code from a Script.
        /// </summary>
        /// <param name="s">The script.</param>
        /// <returns></returns>
        [DebuggerStepThrough]
        public static implicit operator string(Script s)
        {
            return s.Code;
        }
        private object compiledScript = new object();
        /// <summary>
        /// The compiled script.
        /// </summary>
        public object CompiledScript
        {
            [DebuggerStepThrough]
            get { return compiledScript; }
        }

    }
}
