uniform extern float4x4 WorldViewProjection;
float PointScaleX = 1.0f;
float PointScaleY = 1.0f;
float PointScaleZ = 1.0f;
float ScreenHeight = 600.0f;
float MaxPointSize = 128.0f;
float MinPointSize = 1.0f;

struct VS_INPUT
{
	float4 Position  : POSITION0;
	float  PointSize : PSIZE0;
	float4 Color     : COLOR0;
};

struct VS_OUTPUT
{
	float4 Position  : POSITION0;
	float  PointSize : PSIZE0;
	float4 Color     : COLOR0;
};

VS_OUTPUT VertexShader( VS_INPUT input )
{
	VS_OUTPUT output;
	
    output.Position  = mul( input.Position, WorldViewProjection );
    
	// HACK: try to fake perspective by altering the size
    float dist = sqrt( output.Position.x * output.Position.x + output.Position.y * output.Position.y + output.Position.z * output.Position.z );
    float size = input.PointSize * ScreenHeight * sqrt( 1.0f / ( PointScaleX + PointScaleY * dist + PointScaleZ * ( dist * dist ) ) );
    
    if( size < MinPointSize )
		size = MinPointSize;
		
	if( size > MaxPointSize )
		size = MaxPointSize;
    
    output.PointSize = size;
    output.Color     = input.Color;

    return output;
}

struct PS_INPUT
{
    #ifdef XBOX
        float2 TexCoord : SPRITETEXCOORD;
    #else
        float2 TexCoord : TEXCOORD0;
    #endif

    float4 Color : COLOR0;
};

sampler Sampler : register( s0 );                      

float4 PixelShader(PS_INPUT input) : COLOR0
{
    float2 texCoord;

//	#ifdef XBOX
//		texCoord = input.TexCoord.zw;
 //   #else
	    texCoord = input.TexCoord.xy;
//    #endif
    
    return tex2D( Sampler, texCoord ) * input.Color;
}

technique Default
{
	pass Pass0
	{
		VertexShader = compile vs_2_0 VertexShader();
		PixelShader = compile ps_2_0 PixelShader();
	}
}