//#include "Include/Quad.fxh"

uniform extern float3 Ambient : AMBIENT;
uniform extern float3 Contrast : CONTRAST;

uniform extern float4x4 World : WORLD;
uniform extern float4x4 View : VIEW;
uniform extern float4x4 Projection : PROJECTION;


texture DiffuseMapTexture : DIFFUSEMAP;
sampler DiffuseMap = sampler_state
{
	Texture = <DiffuseMapTexture>;
	MipFilter = LINEAR;
	MinFilter = LINEAR;
	MagFilter = LINEAR;
};

texture NormalMapTexture : NORMALMAP;
sampler NormalMap = sampler_state
{
	Texture = <NormalMapTexture>;
	MipFilter = LINEAR;
	MinFilter = LINEAR;
	MagFilter = LINEAR;
};

texture HeightMapTexture : HEIGHTMAP;
sampler HeightMap = sampler_state
{
	Texture = <HeightMapTexture>;
	MipFilter = LINEAR;
	MinFilter = LINEAR;
	MagFilter = LINEAR;
};




struct VS_INPUT
{
	float4 Position : POSITION;
	float3 Normal : NORMAL;
	float3 Tangent : TANGENT;
	float2 TextureCoords : TEXCOORD0;
};

struct VS_OUTPUT
{
  float4 Position : POSITION;
  float3 Normal : TEXCOORD0;
  float3 Tangent : TEXCOORD1;
  float2 TextureCoords : TEXCOORD2;
  float3 VirtualPosition : TEXCOORD3;
};

struct PS_OUTPUT
{
	half4 Colour : COLOR0;  
  half4 NormalTangent : COLOR1;
  half4 Position : COLOR2;
  
};

VS_OUTPUT VertexShaderPass1(in VS_INPUT input)
{
	float4x4 WVP = World*View*Projection;
	VS_OUTPUT output = (VS_OUTPUT)0;
	output.Position = mul(input.Position, WVP);
	output.Normal = mul(input.Normal, WVP);
	output.Tangent = mul(input.Normal, WVP);
	output.TextureCoords = input.TextureCoords;
	output.VirtualPosition = output.Position.xyz;
	return output;
};
PS_OUTPUT PixelShader_ExtractColourNormalTangent(VS_OUTPUT input)
{
	PS_OUTPUT output = (PS_OUTPUT)0;
  
	float3 normal = normalize(input.Normal);
	output.NormalTangent.xy = normal.xy;
  
	float3 tangent = normalize(input.Tangent);  
	output.NormalTangent.zw = tangent.xy;
  
	float2 uv = input.TextureCoords;
	float3 diffuseColour = tex2D(DiffuseMap, uv);
	output.Colour.rgb = diffuseColour;
  
	output.Position.xyz = input.VirtualPosition;
    output.Colour.a = input.TextureCoords.x;
    output.Position.w = input.TextureCoords.y;
  
	return output;
};
technique StandardMaterial
{
	pass PassExtract
	{
		VertexShader = compile vs_1_1 VertexShaderPass1();
		PixelShader = compile ps_3_0 PixelShader_ExtractColourNormalTangent();
	}
};