using System;
using System.Collections.Generic;
using Microsoft.Xna.Framework;
using System.Runtime.InteropServices;
using System.Diagnostics;

namespace Xfinity.Maths
{
    #region FloatOrInt
    /// <summary>
    /// A structure used for accessing the bits of a float.
    /// </summary>
    [StructLayout(LayoutKind.Explicit)]
    public struct FloatOrInt
    {
        /// <summary>
        /// The integer representation.
        /// </summary>
        [FieldOffset(0)]
        public int Integer;
        /// <summary>
        /// The float representation.
        /// </summary>
        [FieldOffset(0)]
        public float Float;
        /// <summary>
        /// The unsigned integer representation.
        /// </summary>
        [FieldOffset(0)]
        public uint UnsignedInteger;
    }
    #endregion
    /// <summary>
    /// Misc. math functions.
    /// </summary>
    /// <remarks>
    /// Don't be a little bitch.
    /// </remarks>
    public static class MathFunctions
    {
        #region Constants
        /// <summary>
        /// One over the square root of pi.
        /// </summary>
        public const float OneOverRootTwoPi = 0.39894228040143267793994605993438f;

        /// <summary>
        /// The gravitational constant (G).
        /// </summary>
        public const double GravitationalConstant = 6.6742e-11;

        /// <summary>
        /// Planks Constant in JouleSeconds.
        /// </summary>
        public const double PlanksConstantInJouleSeconds = 6.6260693e-34;

        /// <summary>
        /// Planks Constant in electron-VoltSeconds.
        /// </summary>
        public const double PlanksConstantInElectronVoltSeconds = 4.13566743e-15;

        /// <summary>
        /// Dirac's Constant in JouleSeconds.
        /// </summary>
        public const double DiracConstantInJouleSeconds = PlanksConstantInJouleSeconds / MathHelper.TwoPi;

        /// <summary>
        /// Dirac's Constant in electron-VoltSeconds.
        /// </summary>
        public const double DiracConstantInElectronVoltSecond = PlanksConstantInElectronVoltSeconds / MathHelper.TwoPi;

        /// <summary>
        /// Boltzmann's Constant.
        /// </summary>
        public const double BoltzmannConstantInJouleKelvins = 1.3806505e-23;

        /// <summary>
        /// Boltzmann's Constant.
        /// </summary>
        public const double BoltzmannConstantInElectronVoltSeconds = 8.617343e-5;

        #endregion
        #region Static variables
        private static Random random = new Random();
        #endregion
        #region Sets
        /// <summary>
        /// Computes the mathematical intersection of two sets.
        /// </summary>
        /// <typeparam name="T">The type.</typeparam>
        /// <param name="a">The first set.</param>
        /// <param name="b">The second set.</param>
        /// <returns>The intersection.</returns>
        public static List<T> Intersection<T>(IList<T> a, IList<T> b)
        {
            List<T> set = new List<T>(Math.Min(a.Count, b.Count));
            foreach (T elementA in a)
            {
                if (b.Contains(elementA))
                {
                    set.Add(elementA);
                }
            }
            set.TrimExcess();
            return set;
        }
        /// <summary>
        /// Computes the mathematical union of two sets.
        /// </summary>
        /// <typeparam name="T">The type.</typeparam>
        /// <param name="a">The first set.</param>
        /// <param name="b">The second set.</param>
        /// <returns>The union.</returns>
        public static List<T> Union<T>(IList<T> a, IList<T> b)
        {
            List<T> union = new List<T>(a.Count + b.Count);
            union.AddRange(a);
            foreach (T element in a)
            {
                if (!b.Contains(element))
                {
                    union.Add(element);
                }
            }
            union.TrimExcess();
            return union;
        }
        /// <summary>
        /// Computes the mathematical disjoint of two sets.
        /// </summary>
        /// <typeparam name="T">The type.</typeparam>
        /// <param name="a">The first set.</param>
        /// <param name="b">The second set.</param>
        /// <returns>The disjoint.</returns>
        public static List<T> Disjoint<T>(IList<T> a, IList<T> b)
        {
            List<T> set = new List<T>(b);
            foreach (T element in a)
            {
                if (set.Contains(element))
                {
                    set.Remove(element);
                }
                else
                {
                    set.Add(element);
                }
            }
            set.TrimExcess();
            return set;
        }
        #endregion
        #region Bit hacks
        /// <summary>
        /// Returns whether the number is odd.
        /// </summary>
        /// <param name="number">The number to check.</param>
        /// <returns>Whether the number is odd.</returns>
        public static bool IsOdd(int number)
        {
            return ((number & 1) == 1);
        }
        /// <summary>
        /// Returns whether the number is odd.
        /// </summary>
        /// <param name="number">The number to check.</param>
        /// <returns>Whether the number is odd.</returns>
        public static bool IsOdd(uint number)
        {
            return ((number & 1) == 1);
        }
        /// <summary>
        /// Returns whether the number is odd.
        /// </summary>
        /// <param name="number">The number to check.</param>
        /// <returns>Whether the number is odd.</returns>
        public static bool IsOdd(byte number)
        {
            return ((number & 1) == 1);
        }
        /// <summary>
        /// Returns whether the number is odd.
        /// </summary>
        /// <param name="number">The number to check.</param>
        /// <returns>Whether the number is odd.</returns>
        public static bool IsOdd(sbyte number)
        {
            return ((number & 1) == 1);
        }
        /// <summary>
        /// Returns whether the number is even.
        /// </summary>
        /// <param name="number">The number to check.</param>
        /// <returns>Whether the number is even.</returns>
        public static bool IsEven(int number)
        {
            return ((number & 1) == 0);
        }
        /// <summary>
        /// Returns whether the number is even.
        /// </summary>
        /// <param name="number">The number to check.</param>
        /// <returns>Whether the number is even.</returns>
        public static bool IsEven(uint number)
        {
            return ((number & 1) == 0);
        }
        /// <summary>
        /// Returns whether the number is even.
        /// </summary>
        /// <param name="number">The number to check.</param>
        /// <returns>Whether the number is even.</returns>
        public static bool IsEven(byte number)
        {
            return ((number & 1) == 0);
        }
        /// <summary>
        /// Returns whether the number is even.
        /// </summary>
        /// <param name="number">The number to check.</param>
        /// <returns>Whether the number is even.</returns>
        public static bool IsEven(sbyte number)
        {
            return ((number & 1) == 0);
        }
        /// <summary>
        /// Gets a bit-for-bit representation of a float, returned as an Int32.
        /// </summary>
        /// <param name="f">The float.</param>
        /// <returns>The Int32.</returns>
        public static unsafe int GetInt32RepresentationOfSingle(float f)
        {
            return *(int*)&f;
        }
        /// <summary>
        /// Gets a bit-for-bit representation of a float, returned as an UInt32.
        /// </summary>
        /// <param name="f">The float.</param>
        /// <returns>The UInt32.</returns>
        public static unsafe uint GetUInt32RepresentationOfSingle(float f)
        {
            return *(uint*)&f;
        }
        /// <summary>
        /// Converts an Int32 back to a Single.
        /// </summary>
        /// <param name="i">The Int32.</param>
        /// <returns>The Single.</returns>
        public static unsafe float GetSingleFromInt32Representation(int i)
        {
            return *(float*)&i;
        }
        /// <summary>
        /// Converts an Int64 back to a Double.
        /// </summary>
        /// <param name="i">The Int64.</param>
        /// <returns>The Double.</returns>
        public static unsafe double GetDoubleFromInt64Representation(long i)
        {
            return *(double*)&i;
        }
        /// <summary>
        /// Gets a bit-for-bit representation of a double, returned as a long.
        /// </summary>
        /// <param name="f">The double.</param>
        /// <returns>The long.</returns>
        public static unsafe long GetInt64RepresentationOfDouble(double f)
        {
            return *(long*)&f;
        }
        /// <summary>
        /// Returns the sign of the number.
        /// </summary>
        public static int SignMask(float f)
        {
            return GetInt32RepresentationOfSingle(f) >> 31;
        }
        /// <summary>
        /// Gets the absolute value of a float.
        /// </summary>
        /// <param name="f">The float.</param>
        /// <returns></returns>
        public static float Abs(float f)
        {
            int i = GetInt32RepresentationOfSingle(f) & 0x7FFFFFFF;
            return GetSingleFromInt32Representation(i);
        }
        /// <summary>
        /// Casts a float to a long
        /// </summary>
        /// <param name="f">The float.</param>
        /// <returns>An Int64.</returns>
        public static long CastFloatToLong(float f)
        {
            double temp = f + (((65536 * 65536.0 * 16) + 32768) * 65536);
            return GetInt64RepresentationOfDouble(f) - 0x80000000U;
        }
        /// <summary>
        /// Gets the integer log2 of the float.
        /// </summary>
        /// <param name="f">The float</param>
        /// <returns>The log.</returns>
        public static uint IntegerLog2(float f)
        {
            Debug.Assert(f > 0);
            uint i = GetUInt32RepresentationOfSingle(f);
            return (i >> 23) - 127;
        }
        /// <summary>
        /// Finds the square root of a number.
        /// </summary>
        /// <param name="f">The number.</param>
        /// <returns>The square root.</returns>
        public static float FastSqrt(float f)
        {
            float ret;
            int i = GetInt32RepresentationOfSingle(f);
            i -= 0x3F800000;
            i >>= 1;
            i += 0x3F800000;
            ret = GetSingleFromInt32Representation(i);
            ret *= (1.5f - (.5f * (f * f * f)));
            return ret;
        }
        /// <summary>
        /// Uses the Lomont Compare algorithm to test if two floats are equal, taking into account bit rot.
        /// </summary>
        /// <param name="a">The first float.</param>
        /// <param name="b">The second float.</param>
        /// <returns>The equality.</returns>
        public static bool Compare(float a, float b)
        {
            return Compare(a, b, 1000);
        }
        /// <summary>
        /// Uses the Lomont Compare algorithm to test if two floats are equal, taking into account bit rot.
        /// </summary>
        /// <param name="af">The first float.</param>
        /// <param name="bf">The second float.</param>
        /// <param name="maxDiff">The tolerance.</param>
        /// <returns>The equality.</returns>
        public static bool Compare(float af, float bf, int maxDiff)
        {
            int ai = GetInt32RepresentationOfSingle(af);
            int bi = GetInt32RepresentationOfSingle(bf);
            int test = SignMask(ai ^ bi);
            int diff = Int32.MinValue - ai;
            diff &= (test) | ((ai & (~test)) - bi);
            int v1 = maxDiff + diff;
            int v2 = maxDiff - diff;
            return (v1 | v2) >= 0;
        }
        /// <summary>
        /// Casts an Int64 to a Single.
        /// </summary>
        /// <param name="i">The integer.</param>
        /// <returns>The float.</returns>
        public static float CastIntToFloat(int i)
        {
            FloatOrInt val = new FloatOrInt();
            val.Integer = i;
            FloatOrInt bias = new FloatOrInt();
            bias.Integer = ((23 + 127) << 23) + (1 << 22);
            val.Integer += bias.Integer;
            val.Float -= bias.Float;
            return val.Float;
        }
        /// <summary>
        /// Casts a float to an int.
        /// </summary>
        /// <param name="f">The float.</param>
        /// <returns>An int.</returns>
        public static int CastFloatToInt(float f)
        {
            f += (1 << 23) + (1 << 22);
            int i = GetInt32RepresentationOfSingle(f);
            i &= (1 << 23) - 1;
            i -= (1 << 22);
            return i;
        }
        /// <summary>
        /// Returns whether a float is negative.
        /// </summary>
        /// <param name="f">The float.</param>
        /// <returns>True for a negative number; false otherwise.</returns>
        public static bool LessThanZero(float f)
        {
            return GetUInt32RepresentationOfSingle(f) > 0x80000000U;
        }
        /// <summary>
        /// Returns whether a float is positive.
        /// </summary>
        /// <param name="f">The float.</param>
        /// <returns>True for a positive number; false otherwise.</returns>
        public static bool GreaterThanZero(float f)
        {
            return GetUInt32RepresentationOfSingle(f) < 0x80000000U;
        }
        /// <summary>
        /// Clamps a float between 0 and 1. Usually faster than every other solution.
        /// </summary>
        /// <param name="f">The float.</param>
        /// <returns>The clamped float.</returns>
        public static float ClampValueToZeroAndOne(float f)
        {
            int s;
            FloatOrInt val = new FloatOrInt();

            //clamp to 0
            val.Float = f; //set floating value
            s = SignMask(val.Integer); // all 1's if sign bit 1 from int value
            val.Integer &= ~s; // 0 if value was negative

            //clamp to 1
            val.Float = 1 - val.Float; //val > 1 goes negative
            //clamp to zero again
            s = SignMask(val.Integer);
            val.Integer &= ~s;
            val.Float = 1 - val.Float; //convert back
            return val.Float;
        }
        /// <summary>
        /// Clamps a value to a maximum of one.
        /// </summary>
        /// <param name="f">The float.</param>
        /// <returns>The modified float.</returns>
        public static float ClampValueToMaximumOfOne(float f)
        {
            int s;
            FloatOrInt val = new FloatOrInt();
            val.Float = 1 - f; //val > 1 goes negative
            //clamp to zero
            s = SignMask(val.Integer);
            val.Integer &= ~s;
            return 1 - val.Float; //convert back
        }
        /// <summary>
        /// Clamps a value to a maximum of a specified number.
        /// </summary>
        /// <param name="f">The float.</param>
        /// <param name="upperLimit">The upper limit.</param>
        /// <returns>The clamped number.</returns>
        public static float ClampValueToZeroAndNumber(float f, float upperLimit)
        {
            int s;
            FloatOrInt val = new FloatOrInt();
            val.Float = upperLimit - f; //val > 1 goes negative
            //clamp to zero
            s = SignMask(val.Integer);
            val.Integer &= ~s;
            return upperLimit - val.Float; //convert back
        }
        /// <summary>
        /// Clamps a value to two other values.
        /// </summary>
        /// <param name="f">The value to clamp.</param>
        /// <param name="lowerLimit">The lower limit.</param>
        /// <param name="upperLimit">The upper limit.</param>
        /// <returns>The clamped value.</returns>
        public static float Clamp(float f, float lowerLimit, float upperLimit)
        {
            float range = upperLimit - lowerLimit;

            FloatOrInt val = new FloatOrInt();
            val.Float = range - (f - lowerLimit); //val > 1 goes negative

            //clamp to zero
            val.Integer &= ~SignMask(val.Integer);

            return range - val.Float + lowerLimit;
        }
        /// <summary>
        /// Clamps a number to non-negative.
        /// </summary>
        /// <param name="f">The number.</param>
        /// <returns>The clamped value.</returns>
        public static float ClampValueToMinimumOfZero(float f)
        {
            int s;
            FloatOrInt val = new FloatOrInt();
            val.Float = f; //set floating value
            s = SignMask(val.Integer); // all 1's if sign bit 1 from int value
            val.Integer &= ~s; // 0 if value was negative
            return val.Float;
        }
        #endregion
        /// <summary>
        /// Calculates a gaussian distribution.
        /// </summary>
        /// <param name="standardDeviation">The standard deviation.</param>
        /// <param name="expectedValue">The expected value.</param>
        /// <param name="x">The point to evaluate at.</param>
        /// <returns>The probability.</returns>
        public static float GaussianDistribution(float standardDeviation, float expectedValue, float x)
        {
            double deviation = OneOverRootTwoPi / standardDeviation;
            double xMinusValue = x - expectedValue;
            double xMVSquared = xMinusValue * xMinusValue;
            double twoDeviation = standardDeviation * 2;
            double twoDSquared = twoDeviation * twoDeviation;
            double power = xMVSquared / twoDSquared;
            return (float)(deviation * Math.Pow(MathHelper.E, power));
        }
        /// <summary>
        /// The Sigmoid function.
        /// </summary>
        /// <param name="x">The position to evaluate.</param>
        /// <param name="midpoint">The midpoint of the curve.</param>
        /// <param name="steepness">The steepness of the curve.</param>
        /// <returns>The height.</returns>
        public static float Sigmoid(float x, float midpoint, float steepness)
        {
            float adjusted = x - midpoint;
            float power = (float)Math.Pow(Math.Pow(MathHelper.E, adjusted), -steepness);
            return 1 / (1 + power);
        }
        /// <summary>
        /// Generates a random direction vector.
        /// </summary>
        /// <returns>A random direction.</returns>
        public static Vector3 RandomDirection()
        {
            // Generate the random values in polar space, otherwise there will be an uneven distribution.
            double rho, theta;
            rho = random.NextDouble() * MathHelper.TwoPi;
            theta = random.NextDouble() * MathHelper.TwoPi;
            double sinRho = Math.Sin(rho);

            float x = (float)(sinRho * Math.Cos(theta));
            float y = (float)(sinRho * Math.Sin(theta));
            float z = (float)(Math.Cos(rho));

            return new Vector3(x, y, z);
        }
        /// <summary>
        /// Evaluates an arbitrary Bezier curve. If there are three points, use a BezierCurve structure instead, as it's more efficient for three points.
        /// </summary>
        /// <param name="points">The control points.</param>
        /// <param name="t">The parameter.</param>
        /// <returns>The point at B(i).</returns>
        public static Vector3 BezierCurve(IList<Vector3> points, float t)
        {
            uint n = (uint)points.Count;
            float oneMinusT = 1 - t;
            Vector3 point;
            point = points[0] * Pow(oneMinusT, n);
            point += points.Count * points[1] * Pow(oneMinusT, n - 1) * t;
            for (uint i = 2; i < points.Count - 2; i++)
            {
                point += BinomialCoefficient(n, i) * points[(int)i] * Pow(oneMinusT, n - i) * Pow(t, i);
            }
            point += points[points.Count - 1] * Pow(t, n);
            return point;
        }
        /// <summary>
        /// Raises a value to an integer power.
        /// </summary>
        /// <param name="x">The value.</param>
        /// <param name="y">The power.</param>
        public static float Pow(float x, uint y)
        {
            float value = 1;
            for (uint i = 0; i < y; i++)
            {
                value *= x;
            }
            return value;
        }
        /// <summary>
        /// Computes the binomial coefficient of two numbers, also known as the choose function.
        /// </summary>
        /// <param name="n">The number of options.</param>
        /// <param name="k">The number of choices.</param>
        /// <returns></returns>
        public static ulong BinomialCoefficient(uint n, uint k)
        {
            uint value = k + 1; ;

            for (uint i = value; i <= n; i++)
            {
                value *= i;
            }

            ulong nMinusKBang = Factorial(n - k);
            return value / (nMinusKBang);
        }

        /// <summary>
        /// Returns the factorial of a number.
        /// </summary>
        /// <param name="n">The number.</param>
        /// <returns>The computed factorial.</returns>
        public static ulong Factorial(uint n)
        {
            ulong factorial = 1;
            for (uint i = 2; i <= n; i++)
            {
                factorial *= i;
            }
            return factorial;
        }
    }
}